@@ -1,11 +1,12 @@
 import os
 import smtplib
+from email.utils import formataddr
 from email.message import EmailMessage
 
 import requests
-from chalice import ChaliceViewError, BadRequestError
+from aws_xray_sdk.core import xray_recorder
 
-from coworks import Blueprint, FileParam
+from coworks import Blueprint, FileParam, entry
 
 
 class Mail(Blueprint):
@@ -30,24 +31,27 @@ class Mail(Blueprint):
             if not self.smtp_passwd:
                 raise EnvironmentError(f'{env_passwd_var_name} not defined in environment.')
 
-    def post_send(self, subject="", from_addr: str = None, to_addrs: [str] = None, cc_addrs: [str] = None,
-                  bcc_addrs: [str] = None, body="",
+    @entry
+    @xray_recorder.capture()
+    def post_send(self, subject="", from_addr: str = None, from_name: str = '', to_addrs: [str] = None,
+                  cc_addrs: [str] = None, bcc_addrs: [str] = None, body="",
                   attachments: [FileParam] = None, attachment_urls: dict = None, subtype="plain", starttls=True):
         """ Send mail.
         To send attachments, add files in the body of the request as multipart/form-data. """
 
         from_addr = from_addr or os.getenv('from_addr')
         if not from_addr:
-            raise BadRequestError("From address not defined (from_addr:str)")
+            return "From address not defined (from_addr:str)", 400
         to_addrs = to_addrs or os.getenv('to_addrs')
         if not to_addrs:
-            raise BadRequestError("To addresses not defined (to_addrs:[str])")
+            return "To addresses not defined (to_addrs:[str])", 400
 
         # Creates email
         try:
+            from_ = formataddr((from_name if from_name else False, from_addr))
             msg = EmailMessage()
             msg['Subject'] = subject
-            msg['From'] = from_addr
+            msg['From'] = from_
             msg['To'] = to_addrs if isinstance(to_addrs, str) else ', '.join(to_addrs)
             if cc_addrs:
                 msg['Cc'] = cc_addrs if isinstance(cc_addrs, str) else ', '.join(cc_addrs)
@@ -60,7 +64,7 @@ class Mail(Blueprint):
                     attachments = [attachments]
                 for attachment in attachments:
                     if not attachment.mime_type:
-                        raise BadRequestError(f"Mime type of the attachment {attachment.file.name} is not defined.")
+                        return f"Mime type of the attachment {attachment.file.name} is not defined.", 400
                     maintype, subtype = attachment.mime_type.split("/")
                     msg.add_attachment(attachment.file.read(), maintype=maintype, subtype=subtype,
                                        filename=attachment.file.name)
@@ -74,10 +78,10 @@ class Mail(Blueprint):
                         msg.add_attachment(attachment, maintype=maintype, subtype=subtype,
                                            filename=attachment_name)
                     else:
-                        raise BadRequestError(f"Failed to download attachment, error {response.status_code}.")
+                        return f"Failed to download attachment, error {response.status_code}.", 400
 
         except Exception as e:
-            raise ChaliceViewError(f"Cannot create email message (Error: {str(e)}).")
+            return f"Cannot create email message (Error: {str(e)}).", 400
 
         # Send emails
         try:
@@ -89,6 +93,6 @@ class Mail(Blueprint):
 
             return f"Mail sent to {msg['To']}"
         except smtplib.SMTPAuthenticationError:
-            raise BadRequestError("Wrong username/password.")
+            return "Wrong username/password : cannot connect.", 400
         except Exception as e:
-            raise ChaliceViewError(f"Cannot send email message (Error: {str(e)}).")
+            return f"Cannot send email message (Error: {str(e)}).", 400
