@@ -19,22 +19,28 @@ class CwsZipArchiver(CwsCommand):
     Uploads also the hash code of this file to be able to determined code changes (used by terraform as a trigger).
     """
 
-    def __init__(self, app=None, name='zip'):
-        super().__init__(app, name=name)
-
     @property
     def options(self):
         return [
+            *super().options,
             click.option('--bucket', '-b', help="Bucket to upload sources zip file to", required=True),
-            click.option('--dry', is_flag=True, help="Doesn't perform upload."),
             click.option('--debug', is_flag=True, help="Print debug logs to stderr."),
+            click.option('--dry', is_flag=True, help="Doesn't perform upload."),
             click.option('--ignore', '-i', multiple=True, help="Ignore pattern."),
             click.option('--key', '-k', help="Sources zip file bucket's name."),
             click.option('--module-name', '-m', multiple=True, help="Python module added from current pyenv."),
             click.option('--profile_name', '-p', required=True, help="AWS credential profile."),
         ]
 
-    def _execute(self, *, project_dir, module, bucket, key, profile_name, module_name, dry, debug, ignore, **options):
+    @classmethod
+    def multi_execute(cls, project_dir, workspace, execution_list):
+        for command, options in execution_list:
+            command._zip(**options)
+
+    def __init__(self, app=None, name='zip'):
+        super().__init__(app, name=name)
+
+    def _zip(self, *, project_dir, module, bucket, key, profile_name, module_name, dry, debug, ignore, **options):
         aws_s3_session = aws.AwsS3Session(profile_name=profile_name)
         module_name = module_name or []
 
